@@ -1,4 +1,5 @@
 # Copyright (C) 2008 One Laptop Per Child
+# Copyright (C) 2009 Tomeu Vizoso
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,13 +20,13 @@ import logging
 from gettext import gettext as _
 
 import gobject
-import gtk
-import hippo
+import pango
 import gconf
+import gtk
 
 from sugar import util
 from sugar.graphics import style
-from sugar.graphics.icon import CanvasIcon, Icon
+from sugar.graphics.icon import Icon, CellRendererIcon
 from sugar.graphics.xocolor import XoColor
 from sugar.graphics.menuitem import MenuItem
 from sugar.graphics.alert import Alert
@@ -36,6 +37,264 @@ from jarabe.model import bundleregistry
 from jarabe.view.palettes import ActivityPalette
 from jarabe.view import launcher
 
+class ActivitiesTreeView(gtk.TreeView):
+    __gtype_name__ = 'SugarActivitiesTreeView'
+
+    __gsignals__ = {
+        'erase-activated' : (gobject.SIGNAL_RUN_FIRST,
+                             gobject.TYPE_NONE, ([str]))
+    }
+
+    def __init__(self):
+        gobject.GObject.__init__(self)
+
+        self._query = ''
+
+        self.modify_base(gtk.STATE_NORMAL, style.COLOR_WHITE.get_gdk_color())
+        self.set_headers_visible(False)
+        selection = self.get_selection()
+        selection.set_mode(gtk.SELECTION_NONE)
+
+        model = ListModel()
+        model.set_visible_func(self.__model_visible_cb)
+        self.set_model(model)
+
+        cell_favorite = CellRendererFavorite(self)
+        cell_favorite.connect('clicked', self.__favorite_clicked_cb)
+
+        column = gtk.TreeViewColumn()
+        column.pack_start(cell_favorite)
+        column.set_cell_data_func(cell_favorite, self.__favorite_set_data_cb)
+        self.append_column(column)
+
+        cell_icon = CellRendererActivityIcon(self)
+        cell_icon.connect('erase-activated', self.__erase_activated_cb)
+        cell_icon.connect('clicked', self.__icon_clicked_cb)
+
+        column = gtk.TreeViewColumn()
+        column.pack_start(cell_icon)
+        column.add_attribute(cell_icon, 'file-name', ListModel.COLUMN_ICON)
+        self.append_column(column)
+
+        cell_text = gtk.CellRendererText()
+        cell_text.props.ellipsize = pango.ELLIPSIZE_MIDDLE
+        cell_text.props.ellipsize_set = True
+
+        column = gtk.TreeViewColumn()
+        column.props.sizing = gtk.TREE_VIEW_COLUMN_GROW_ONLY
+        column.props.expand = True
+        column.set_sort_column_id(ListModel.COLUMN_TITLE)
+        column.pack_start(cell_text)
+        column.add_attribute(cell_text, 'markup', ListModel.COLUMN_TITLE)
+        self.append_column(column)
+
+        cell_text = gtk.CellRendererText()
+        cell_text.props.xalign = 1
+
+        column = gtk.TreeViewColumn()
+        column.set_alignment(1)
+        column.props.sizing = gtk.TREE_VIEW_COLUMN_GROW_ONLY
+        column.props.resizable = True
+        column.props.reorderable = True
+        column.props.expand = True
+        column.set_sort_column_id(ListModel.COLUMN_VERSION)
+        column.pack_start(cell_text)
+        column.add_attribute(cell_text, 'text', ListModel.COLUMN_VERSION_TEXT)
+        self.append_column(column)
+
+        cell_text = gtk.CellRendererText()
+        cell_text.props.xalign = 1
+
+        column = gtk.TreeViewColumn()
+        column.set_alignment(1)
+        column.props.sizing = gtk.TREE_VIEW_COLUMN_GROW_ONLY
+        column.props.resizable = True
+        column.props.reorderable = True
+        column.props.expand = True
+        column.set_sort_column_id(ListModel.COLUMN_DATE)
+        column.pack_start(cell_text)
+        column.add_attribute(cell_text, 'text', ListModel.COLUMN_DATE_TEXT)
+        self.append_column(column)
+
+        self.set_search_column(ListModel.COLUMN_TITLE)
+
+    def __erase_activated_cb(self, cell_renderer, bundle_id):
+        self.emit('erase-activated', bundle_id)
+
+    def __favorite_set_data_cb(self, column, cell, model, tree_iter):
+        favorite = model[tree_iter][ListModel.COLUMN_FAVORITE]
+        if favorite:
+            client = gconf.client_get_default()
+            color = XoColor(client.get_string('/desktop/sugar/user/color'))
+            cell.props.xo_color = color
+        else:
+            cell.props.xo_color = None
+
+    def __favorite_clicked_cb(self, cell, path):
+        row = self.get_model()[path]
+        registry = bundleregistry.get_registry()
+        registry.set_bundle_favorite(row[ListModel.COLUMN_BUNDLE_ID],
+                                     row[ListModel.COLUMN_VERSION],
+                                     not row[ListModel.COLUMN_FAVORITE])
+
+    def __icon_clicked_cb(self, cell, path):
+        row = self.get_model()[path]
+
+        registry = bundleregistry.get_registry()
+        bundle = registry.get_bundle(row[ListModel.COLUMN_BUNDLE_ID])
+
+        activity_id = activityfactory.create_activity_id()
+
+        client = gconf.client_get_default()
+        xo_color = XoColor(client.get_string('/desktop/sugar/user/color'))
+
+        launcher.add_launcher(activity_id, bundle.get_icon(), xo_color)
+        activityfactory.create(bundle, ActivityHandle(activity_id))
+
+    def set_filter(self, query):
+        self._query = query.lower()
+        self.get_model().refilter()
+
+    def __model_visible_cb(self, model, tree_iter):
+        title = model[tree_iter][ListModel.COLUMN_TITLE]
+        return title is not None and title.lower().find(self._query) > -1
+
+class ListModel(gtk.TreeModelSort):
+    __gtype_name__ = 'SugarListModel'
+
+    COLUMN_BUNDLE_ID = 0
+    COLUMN_FAVORITE = 1
+    COLUMN_ICON = 2
+    COLUMN_TITLE = 3
+    COLUMN_VERSION = 4
+    COLUMN_VERSION_TEXT = 5
+    COLUMN_DATE = 6
+    COLUMN_DATE_TEXT = 7
+
+    def __init__(self):
+        self._model = gtk.ListStore(str, bool, str, str, int, str, int, str)
+        self._model_filter = self._model.filter_new()
+        gtk.TreeModelSort.__init__(self, self._model_filter)
+
+        gobject.idle_add(self.__connect_to_bundle_registry_cb)
+
+    def __connect_to_bundle_registry_cb(self):
+        registry = bundleregistry.get_registry()
+        for info in registry:
+            self._add_activity(info)
+        registry.connect('bundle-added', self.__activity_added_cb)
+        registry.connect('bundle-changed', self.__activity_changed_cb)
+        registry.connect('bundle-removed', self.__activity_removed_cb)
+
+    def __activity_added_cb(self, activity_registry, activity_info):
+        self._add_activity(activity_info)
+
+    def __activity_changed_cb(self, activity_registry, activity_info):
+        bundle_id = activity_info.get_bundle_id()
+        version = activity_info.get_activity_version()
+        favorite = activity_registry.is_bundle_favorite(bundle_id, version)
+        for row in self._model:
+            if row[ListModel.COLUMN_BUNDLE_ID] == bundle_id and \
+                    row[ListModel.COLUMN_VERSION] == version:
+                row[ListModel.COLUMN_FAVORITE] = favorite
+                return
+
+    def __activity_removed_cb(self, activity_registry, activity_info):
+        bundle_id = activity_info.get_bundle_id()
+        version = activity_info.get_activity_version()
+        for row in self._model:
+            if row[ListModel.COLUMN_BUNDLE_ID] == bundle_id and \
+                    row[ListModel.COLUMN_VERSION] == version:
+                self._model.remove(row.iter)
+                return
+
+    def _add_activity(self, activity_info):
+        if activity_info.get_bundle_id() == 'org.laptop.JournalActivity':
+            return
+
+        timestamp = activity_info.get_installation_time()
+        version = activity_info.get_activity_version()
+
+        registry = bundleregistry.get_registry()
+        favorite = registry.is_bundle_favorite(activity_info.get_bundle_id(),
+                                               version)
+
+        tag_list = activity_info.get_tags()
+        if tag_list is None or not tag_list:
+            title = '<b>%s</b>' % activity_info.get_name()
+        else:
+            tags = ', '.join(tag_list)
+            title = '<b>%s</b>\n' \
+                    '<span style="italic" weight="light">%s</span>' % \
+                            (activity_info.get_name(), tags)
+
+        self._model.append([activity_info.get_bundle_id(),
+                            favorite,
+                            activity_info.get_icon(),
+                            title,
+                            version,
+                            _('Version %s') % version,
+                            timestamp,
+                            util.timestamp_to_elapsed_string(timestamp)])
+
+    def set_visible_func(self, func):
+        self._model_filter.set_visible_func(func)
+
+    def refilter(self):
+        self._model_filter.refilter()
+
+class CellRendererFavorite(CellRendererIcon):
+    __gtype_name__ = 'SugarCellRendererFavorite'
+
+    def __init__(self, tree_view):
+        CellRendererIcon.__init__(self, tree_view)
+
+        self.props.width = style.GRID_CELL_SIZE
+        self.props.height = style.GRID_CELL_SIZE
+        self.props.size = style.SMALL_ICON_SIZE
+        self.props.icon_name = 'emblem-favorite'
+        self.props.mode = gtk.CELL_RENDERER_MODE_ACTIVATABLE
+        self.props.prelit_stroke_color = style.COLOR_BUTTON_GREY.get_svg()
+        self.props.prelit_fill_color = style.COLOR_BUTTON_GREY.get_svg()
+
+class CellRendererActivityIcon(CellRendererIcon):
+    __gtype_name__ = 'SugarCellRendererActivityIcon'
+
+    __gsignals__ = {
+        'erase-activated' : (gobject.SIGNAL_RUN_FIRST,
+                             gobject.TYPE_NONE, ([str]))
+    }
+
+    def __init__(self, tree_view):
+        CellRendererIcon.__init__(self, tree_view)
+
+        self.props.width = style.GRID_CELL_SIZE
+        self.props.height = style.GRID_CELL_SIZE
+        self.props.size = style.STANDARD_ICON_SIZE
+        self.props.stroke_color = style.COLOR_BUTTON_GREY.get_svg()
+        self.props.fill_color = style.COLOR_TRANSPARENT.get_svg()
+        self.props.mode = gtk.CELL_RENDERER_MODE_ACTIVATABLE
+
+        client = gconf.client_get_default()
+        prelit_color = XoColor(client.get_string('/desktop/sugar/user/color'))
+        self.props.prelit_stroke_color = prelit_color.get_stroke_color()
+        self.props.prelit_fill_color = prelit_color.get_fill_color()
+
+        self._tree_view = tree_view
+
+    def create_palette(self):
+        model = self._tree_view.get_model()
+        row = model[self.props.palette_invoker.path]
+        bundle_id = row[ListModel.COLUMN_BUNDLE_ID]
+
+        registry = bundleregistry.get_registry()
+        palette = ActivityListPalette(registry.get_bundle(bundle_id))
+        palette.connect('erase-activated', self.__erase_activated_cb)
+        return palette
+
+    def __erase_activated_cb(self, palette, bundle_id):
+        self.emit('erase-activated', bundle_id)
+
 class ActivitiesList(gtk.VBox):
     __gtype_name__ = 'SugarActivitiesList'
 
@@ -51,80 +310,15 @@ class ActivitiesList(gtk.VBox):
         self.pack_start(scrolled_window)
         scrolled_window.show()
 
-        canvas = hippo.Canvas()
-        scrolled_window.add_with_viewport(canvas)
-        scrolled_window.child.set_shadow_type(gtk.SHADOW_NONE)
-        canvas.show()
+        self._tree_view = ActivitiesTreeView()
+        self._tree_view.connect('erase-activated', self.__erase_activated_cb)
+        scrolled_window.add(self._tree_view)
+        self._tree_view.show()
 
         self._alert = None
-        self._query = ''
-        self._box = hippo.CanvasBox()
-        self._box.props.background_color = style.COLOR_WHITE.get_int()
-        canvas.set_root(self._box)
-
-        gobject.idle_add(self.__connect_to_bundle_registry_cb)
-
-    def __connect_to_bundle_registry_cb(self):
-        registry = bundleregistry.get_registry()
-        for info in registry:
-            self._add_activity(info)
-        registry.connect('bundle-added', self.__activity_added_cb)
-        registry.connect('bundle-removed', self.__activity_removed_cb)
-
-    def __activity_added_cb(self, activity_registry, activity_info):
-        self._add_activity(activity_info)
-
-    def __activity_removed_cb(self, activity_registry, activity_info):
-        for entry in self._box.get_children():
-            if entry.get_bundle_id() == activity_info.get_bundle_id() and \
-                    entry.get_version() == activity_info.get_activity_version():
-                self._box.remove(entry)
-                return
-
-    def _compare_activities(self, entry_a, entry_b):
-        return entry_b.get_installation_time() - entry_a.get_installation_time()
-
-    def _add_activity(self, activity_info):
-        if activity_info.get_bundle_id() == 'org.laptop.JournalActivity':
-            return
-        entry = ActivityEntry(activity_info)
-        entry.icon.connect('erase-activated', self.__erase_activated_cb)
-        self._box.insert_sorted(entry, 0, self._compare_activities)
-        entry.set_visible(entry.matches(self._query))
-
-    def __erase_activated_cb(self, activity_icon, bundle_id):
-        registry = bundleregistry.get_registry()
-        activity_info = registry.get_bundle(bundle_id)
-
-        alert = Alert()
-        alert.props.title = _('Confirm erase')
-        alert.props.msg = \
-                _('Confirm erase: Do you want to permanently erase %s?') \
-                % activity_info.get_name()
-
-        cancel_icon = Icon(icon_name='dialog-cancel')
-        alert.add_button(gtk.RESPONSE_CANCEL, _('Keep'), cancel_icon)
-
-        erase_icon = Icon(icon_name='dialog-ok')
-        alert.add_button(gtk.RESPONSE_OK, _('Erase'), erase_icon)
-
-        alert.connect('response', self.__erase_confirmation_dialog_response_cb,
-                bundle_id)
-
-        self.add_alert(alert)
-
-    def __erase_confirmation_dialog_response_cb(self, alert, response_id,
-                                                bundle_id):
-        self.remove_alert()
-        if response_id == gtk.RESPONSE_OK:
-            registry = bundleregistry.get_registry()
-            bundle = registry.get_bundle(bundle_id)
-            registry.uninstall(bundle)
 
     def set_filter(self, query):
-        self._query = query
-        for entry in self._box.get_children():
-            entry.set_visible(entry.matches(query))
+        self._tree_view.set_filter(query)
 
     def __key_press_event_cb(self, scrolled_window, event):
         keyname = gtk.gdk.keyval_name(event.keyval)
@@ -155,198 +349,34 @@ class ActivitiesList(gtk.VBox):
         self.remove(self._alert)
         self._alert = None
 
-class ActivityIcon(CanvasIcon):
-    __gtype_name__ = 'SugarListActivityIcon'
-
-    __gsignals__ = {
-        'erase-activated' : (gobject.SIGNAL_RUN_FIRST,
-                             gobject.TYPE_NONE, ([str]))
-    }
-
-    def __init__(self, activity_info):
-        CanvasIcon.__init__(self, size=style.STANDARD_ICON_SIZE, cache=True,
-                            file_name=activity_info.get_icon())
-        self._activity_info = activity_info
-        self._uncolor()
-        self.connect('hovering-changed', self.__hovering_changed_event_cb)
-        self.connect('button-release-event', self.__button_release_event_cb)
-
-        client = gconf.client_get_default()
-        self._xocolor = XoColor(client.get_string("/desktop/sugar/user/color"))
-
-    def create_palette(self):
-        palette = ActivityListPalette(self._activity_info)
-        palette.connect('erase-activated', self.__erase_activated_cb)
-        return palette
-
-    def __erase_activated_cb(self, palette, bundle_id):
-        self.emit('erase-activated', self._activity_info.get_bundle_id())
-
-    def _color(self):
-        self.props.xo_color = self._xocolor
-
-    def _uncolor(self):
-        self.props.stroke_color = style.COLOR_BUTTON_GREY.get_svg()
-        self.props.fill_color = style.COLOR_TRANSPARENT.get_svg()
-
-    def __hovering_changed_event_cb(self, icon, hovering):
-        if hovering:
-            self._color()
-        else:
-            self._uncolor()
-
-    def __button_release_event_cb(self, icon, event):
-        self.palette.popdown(immediate=True)
-        self._uncolor()
-
-class ActivityEntry(hippo.CanvasBox, hippo.CanvasItem):
-    __gtype_name__ = 'SugarActivityEntry'
-
-    _TITLE_COL_WIDTH   = style.GRID_CELL_SIZE * 3
-    _VERSION_COL_WIDTH = style.GRID_CELL_SIZE * 3
-    _DATE_COL_WIDTH    = style.GRID_CELL_SIZE * 5
-
-    def __init__(self, activity_info):
-        hippo.CanvasBox.__init__(self, spacing=style.DEFAULT_SPACING,
-                                 padding_top=style.DEFAULT_PADDING,
-                                 padding_bottom=style.DEFAULT_PADDING,
-                                 padding_left=style.DEFAULT_PADDING * 2,
-                                 padding_right=style.DEFAULT_PADDING * 2,
-                                 box_height=style.GRID_CELL_SIZE,
-                                 orientation=hippo.ORIENTATION_HORIZONTAL)
-
+    def __erase_activated_cb(self, tree_view, bundle_id):
         registry = bundleregistry.get_registry()
-        registry.connect('bundle-changed', self.__activity_changed_cb)
+        activity_info = registry.get_bundle(bundle_id)
 
-        self._bundle = activity_info
-        self._bundle_id = activity_info.get_bundle_id()
-        self._version = activity_info.get_activity_version()
-        self._favorite = registry.is_bundle_favorite(self._bundle_id,
-                                                     self._version)
-        self._title = activity_info.get_name()
-        self._installation_time = activity_info.get_installation_time()
+        alert = Alert()
+        alert.props.title = _('Confirm erase')
+        alert.props.msg = \
+                _('Confirm erase: Do you want to permanently erase %s?') \
+                % activity_info.get_name()
 
-        self._favorite_icon = FavoriteIcon(self._favorite)
-        self._favorite_icon.connect('notify::favorite',
-                                    self.__favorite_changed_cb)
-        self.append(self._favorite_icon)
+        cancel_icon = Icon(icon_name='dialog-cancel')
+        alert.add_button(gtk.RESPONSE_CANCEL, _('Keep'), cancel_icon)
 
-        self.icon = ActivityIcon(activity_info)
-        self.icon.connect('button-release-event',
-                          self.__icon_button_release_event_cb)
-        self.append(self.icon)
+        erase_icon = Icon(icon_name='dialog-ok')
+        alert.add_button(gtk.RESPONSE_OK, _('Erase'), erase_icon)
 
-        if gtk.widget_get_default_direction() == gtk.TEXT_DIR_RTL:
-            align = hippo.ALIGNMENT_END
-        else:
-            align = hippo.ALIGNMENT_START
+        alert.connect('response', self.__erase_confirmation_dialog_response_cb,
+                bundle_id)
 
-        title = hippo.CanvasText(text=activity_info.get_name(),
-                                 xalign=align,
-                                 font_desc=style.FONT_BOLD.get_pango_desc(),
-                                 box_width=ActivityEntry._TITLE_COL_WIDTH)
-        self.append(title)
-
-        version = hippo.CanvasText(text=activity_info.get_activity_version(),
-                                   xalign=hippo.ALIGNMENT_END,
-                                   font_desc=style.FONT_NORMAL.get_pango_desc(),
-                                   box_width=ActivityEntry._VERSION_COL_WIDTH)
-        self.append(version)
-
-        expander = hippo.CanvasBox()
-        self.append(expander, hippo.PACK_EXPAND)
-
-        timestamp = activity_info.get_installation_time()
-        date = hippo.CanvasText(
-                text=util.timestamp_to_elapsed_string(timestamp),
-                xalign=align,
-                font_desc=style.FONT_NORMAL.get_pango_desc(),
-                box_width=ActivityEntry._DATE_COL_WIDTH)
-        self.append(date)
-
-        if gtk.widget_get_default_direction() == gtk.TEXT_DIR_RTL:
-            self.reverse()
-
-    def __favorite_changed_cb(self, favorite_icon, pspec):
-        registry = bundleregistry.get_registry()
-        registry.set_bundle_favorite(self._bundle_id, self._version,
-                                       favorite_icon.props.favorite)
-
-    def __activity_changed_cb(self, activity_registry, activity_info):
-        if self._bundle_id == activity_info.get_bundle_id() and \
-                self._version == activity_info.get_activity_version():
-            self._title = activity_info.get_name()
+        self.add_alert(alert)
 
+    def __erase_confirmation_dialog_response_cb(self, alert, response_id,
+                                                bundle_id):
+        self.remove_alert()
+        if response_id == gtk.RESPONSE_OK:
             registry = bundleregistry.get_registry()
-            self._favorite = registry.is_bundle_favorite(self._bundle_id,
-                                                         self._version)
-
-            self._favorite_icon.props.favorite = self._favorite
-
-    def __icon_button_release_event_cb(self, icon, event):
-        activity_id = activityfactory.create_activity_id()
-
-        client = gconf.client_get_default()
-        xo_color = XoColor(client.get_string('/desktop/sugar/user/color'))
-
-        launcher.add_launcher(activity_id,
-                              self._bundle.get_icon(),
-                              xo_color)
-
-        activityfactory.create(self._bundle, ActivityHandle(activity_id))
-
-    def get_bundle_id(self):
-        return self._bundle_id
-
-    def get_version(self):
-        return self._version
-
-    def get_installation_time(self):
-        return self._installation_time
-
-    def matches(self, query):
-        if not query:
-            return True
-        return self._title.lower().find(query) > -1
-
-class FavoriteIcon(CanvasIcon):
-    def __init__(self, favorite):
-        CanvasIcon.__init__(self, icon_name='emblem-favorite',
-                            box_width=style.GRID_CELL_SIZE*3/5,
-                            size=style.SMALL_ICON_SIZE)
-        self._favorite = None
-        self.set_favorite(favorite)
-        self.connect('button-release-event', self.__release_event_cb)
-        self.connect('motion-notify-event', self.__motion_notify_event_cb)
-
-    def set_favorite(self, favorite):
-        if favorite == self._favorite:
-            return
-
-        self._favorite = favorite
-        if favorite:
-            client = gconf.client_get_default()
-            color = XoColor(client.get_string('/desktop/sugar/user/color'))
-            self.props.xo_color = color
-        else:
-            self.props.stroke_color = style.COLOR_BUTTON_GREY.get_svg()
-            self.props.fill_color = style.COLOR_WHITE.get_svg()
-
-    def get_favorite(self):
-        return self._favorite
-
-    favorite = gobject.property(
-        type=bool, default=False, getter=get_favorite, setter=set_favorite)
-
-    def __release_event_cb(self, icon, event):
-        self.props.favorite = not self.props.favorite
-
-    def __motion_notify_event_cb(self, icon, event):
-        if not self._favorite:
-            if event.detail == hippo.MOTION_DETAIL_ENTER:
-                icon.props.fill_color = style.COLOR_BUTTON_GREY.get_svg()
-            elif event.detail == hippo.MOTION_DETAIL_LEAVE:
-                icon.props.fill_color = style.COLOR_TRANSPARENT.get_svg()
+            bundle = registry.get_bundle(bundle_id)
+            registry.uninstall(bundle)
 
 class ActivityListPalette(ActivityPalette):
     __gtype_name__ = 'SugarActivityListPalette'
@@ -375,7 +405,14 @@ class ActivityListPalette(ActivityPalette):
         self.menu.append(self._favorite_item)
         self._favorite_item.show()
 
-        self._add_erase_option(registry, activity_info)
+        if activity_info.is_user_activity():
+            menu_item = MenuItem(_('Erase'), 'list-remove')
+            menu_item.connect('activate', self.__erase_activate_cb)
+            self.menu.append(menu_item)
+            menu_item.show()
+
+            if not os.access(activity_info.get_path(), os.W_OK):
+                menu_item.props.sensitive = False
 
         registry = bundleregistry.get_registry()
         self._activity_changed_sid = registry.connect('bundle_changed',
@@ -384,17 +421,6 @@ class ActivityListPalette(ActivityPalette):
 
         self.connect('destroy', self.__destroy_cb)
 
-    def _add_erase_option(self, registry, activity_info):
-        menu_item = MenuItem(_('Erase'), 'list-remove')
-        menu_item.connect('activate', self.__erase_activate_cb)
-        self.menu.append(menu_item)
-        menu_item.show()
-
-        if not os.access(activity_info.get_path(), os.W_OK) or \
-                registry.is_activity_protected(self._bundle_id):
-            menu_item.props.sensitive = False
-
-
     def __destroy_cb(self, palette):
         self.disconnect(self._activity_changed_sid)
 
