@@ -18,7 +18,7 @@
 import logging
 from gettext import gettext as _
 import sys
-import traceback 
+import traceback
 import uuid
 
 import gtk
@@ -27,9 +27,6 @@ import statvfs
 import os
 
 from sugar.graphics.window import Window
-from sugar.graphics.alert import Alert
-from sugar.graphics.icon import Icon
-
 from sugar.bundle.bundle import ZipExtractException, RegistrationException
 from sugar import env
 from sugar.activity import activityfactory
@@ -70,7 +67,7 @@ class JournalActivityDBusService(dbus.service.Object):
         logging.debug('Trying to show object %s', object_id)
 
         if self._parent.show_object(object_id):
-            self._parent.present()
+            self._parent.reveal()
 
     def _chooser_response_cb(self, chooser, response_id, chooser_id):
         logging.debug('JournalActivityDBusService._chooser_response_cb')
@@ -134,25 +131,13 @@ class JournalActivity(Window):
         model.updated.connect(self.__model_updated_cb)
         model.deleted.connect(self.__model_deleted_cb)
 
-        self._dbus_service = JournalActivityDBusService(self)        
+        self._dbus_service = JournalActivityDBusService(self)
 
         self.iconify()
-        
+
         self._critical_space_alert = None
         self._check_available_space()
 
-    def __volume_error_cb(self, gobject, message, severity):
-        alert = Alert(title=severity, msg=message)
-        icon = Icon(icon_name='dialog-ok')
-        alert.add_button(gtk.RESPONSE_OK, _('Ok'), icon)
-        icon.show()
-        alert.connect('response', self.__alert_response_cb)
-        self.add_alert(alert)
-        alert.show()
-
-    def __alert_response_cb(self, alert, response_id):
-        self.remove_alert(alert)
-
     def __realize_cb(self, window):
         wm.set_bundle_id(window.window, _BUNDLE_ID)
         activity_id = activityfactory.create_activity_id()
@@ -176,8 +161,6 @@ class JournalActivity(Window):
         self._volumes_toolbar = VolumesToolbar()
         self._volumes_toolbar.connect('volume-changed',
                                       self.__volume_changed_cb)
-        self._volumes_toolbar.connect('volume-error',
-                                      self.__volume_error_cb)
         self._main_view.pack_start(self._volumes_toolbar, expand=False)
 
         search_toolbar = self._main_toolbox.search_toolbar
@@ -188,8 +171,8 @@ class JournalActivity(Window):
         self._secondary_view = gtk.VBox()
 
         self._detail_toolbox = DetailToolbox()
-        self._detail_toolbox.entry_toolbar.connect('volume-error',
-                                                   self.__volume_error_cb)
+        entry_toolbar = self._detail_toolbox.entry_toolbar
+
         self._detail_view = DetailView()
         self._detail_view.connect('go-back-clicked', self.__go_back_clicked_cb)
         self._secondary_view.pack_end(self._detail_view)
@@ -200,8 +183,8 @@ class JournalActivity(Window):
         if keyname == 'Escape':
             self.show_main_view()
 
-    def __detail_clicked_cb(self, list_view, entry):
-        self._show_secondary_view(entry.metadata)
+    def __detail_clicked_cb(self, list_view, object_id):
+        self._show_secondary_view(object_id)
 
     def __clear_clicked_cb(self, list_view):
         self._main_toolbox.search_toolbar.clear_query()
@@ -214,24 +197,23 @@ class JournalActivity(Window):
         self.show_main_view()
 
     def show_main_view(self):
-        if self.toolbox != self._main_toolbox:
-            self.set_toolbox(self._main_toolbox)
+        if self.toolbar_box != self._main_toolbox:
+            self.set_toolbar_box(self._main_toolbox)
             self._main_toolbox.show()
 
         if self.canvas != self._main_view:
             self.set_canvas(self._main_view)
             self._main_view.show()
 
-    def _show_secondary_view(self, metadata):
-        # Need to get the full set of properties
-        metadata = model.get(metadata['uid'])
+    def _show_secondary_view(self, object_id):
+        metadata = model.get(object_id)
         try:
             self._detail_toolbox.entry_toolbar.set_metadata(metadata)
         except Exception:
             logging.error('Exception while displaying entry:\n' + \
                 ''.join(traceback.format_exception(*sys.exc_info())))
 
-        self.set_toolbox(self._detail_toolbox)
+        self.set_toolbar_box(self._detail_toolbox)
         self._detail_toolbox.show()
 
         try:
@@ -248,11 +230,11 @@ class JournalActivity(Window):
         if metadata is None:
             return False
         else:
-            self._show_secondary_view(metadata)
+            self._show_secondary_view(object_id)
             return True
 
     def __volume_changed_cb(self, volume_toolbar, mount_point):
-        logging.debug('Selected volume: %r.' % mount_point)
+        logging.debug('Selected volume: %r.', mount_point)
         self._main_toolbox.search_toolbar.set_mount_point(mount_point)
         self._main_toolbox.set_current_toolbar(0)
 
@@ -309,8 +291,7 @@ class JournalActivity(Window):
         try:
             registry.install(bundle)
         except (ZipExtractException, RegistrationException):
-            logging.warning('Could not install bundle %s:\n%s' % \
-                            (bundle.get_path(), traceback.format_exc()))
+            logging.exception('Could not install bundle %s', bundle.get_path())
             return
 
         metadata['bundle_id'] = bundle.get_bundle_id()
@@ -321,22 +302,22 @@ class JournalActivity(Window):
         search_toolbar.give_entry_focus()
 
     def __window_state_event_cb(self, window, event):
-        logging.debug('window_state_event_cb %r' % self)
+        logging.debug('window_state_event_cb %r', self)
         if event.changed_mask & gtk.gdk.WINDOW_STATE_ICONIFIED:
             state = event.new_window_state
             visible = not state & gtk.gdk.WINDOW_STATE_ICONIFIED
             self._list_view.set_is_visible(visible)
 
     def __visibility_notify_event_cb(self, window, event):
-        logging.debug('visibility_notify_event_cb %r' % self)
+        logging.debug('visibility_notify_event_cb %r', self)
         visible = event.state != gtk.gdk.VISIBILITY_FULLY_OBSCURED
         self._list_view.set_is_visible(visible)
 
     def _check_available_space(self):
         ''' Check available space on device
 
-            If the available space is below 50MB an alert will be 
-            shown which encourages to delete old journal entries. 
+            If the available space is below 50MB an alert will be
+            shown which encourages to delete old journal entries.
         '''
 
         if self._critical_space_alert:
@@ -345,13 +326,13 @@ class JournalActivity(Window):
         free_space = stat[statvfs.F_BSIZE] * stat[statvfs.F_BAVAIL]
         if free_space < _SPACE_TRESHOLD:
             self._critical_space_alert = ModalAlert()
-            self._critical_space_alert.connect('destroy', 
+            self._critical_space_alert.connect('destroy',
                                                self.__alert_closed_cb)
             self._critical_space_alert.show()
-        
-    def __alert_closed_cb(self, data):  
+
+    def __alert_closed_cb(self, data):
         self.show_main_view()
-        self.present()
+        self.reveal()
         self._critical_space_alert = None
 
     def set_active_volume(self, mount):
@@ -360,7 +341,7 @@ class JournalActivity(Window):
     def focus_search(self):
         """Become visible and give focus to the search entry
         """
-        self.present()
+        self.reveal()
         self.show_main_view()
         self.search_grab_focus()
 
