@@ -15,6 +15,7 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 import logging
+from gettext import gettext as _
 
 import gtk
 import hippo
@@ -28,21 +29,49 @@ from sugar.graphics.xocolor import XoColor
 from jarabe.model import shell
 from jarabe.view.pulsingicon import CanvasPulsingIcon
 
+
 class LaunchWindow(gtk.Window):
+
     def __init__(self, activity_id, icon_path, icon_color):
         gobject.GObject.__init__(self)
 
         self.props.type_hint = gtk.gdk.WINDOW_TYPE_HINT_NORMAL
         self.props.decorated = False
+        self.modify_bg(gtk.STATE_NORMAL, style.COLOR_WHITE.get_gdk_color())
 
-        canvas = hippo.Canvas()
-        canvas.modify_bg(gtk.STATE_NORMAL, style.COLOR_WHITE.get_gdk_color())
-        self.add(canvas)
+        canvas = gtk.VBox()
         canvas.show()
+        self.add(canvas)
+
+        bar_size = gtk.gdk.screen_height() / 5 * 2
+
+        header = gtk.VBox()
+        header.set_size_request(-1, bar_size)
+        header.show()
+        canvas.pack_start(header, expand=False)
 
         self._activity_id = activity_id
         self._box = LaunchBox(activity_id, icon_path, icon_color)
-        canvas.set_root(self._box)
+        box = hippo.Canvas()
+        box.modify_bg(gtk.STATE_NORMAL, style.COLOR_WHITE.get_gdk_color())
+        box.set_root(self._box)
+        box.show()
+        canvas.pack_start(box)
+
+        footer = gtk.VBox(spacing=style.DEFAULT_SPACING)
+        footer.set_size_request(-1, bar_size)
+        footer.show()
+        canvas.pack_end(footer, expand=False)
+
+        self.error_text = gtk.Label()
+        self.error_text.props.use_markup = True
+        footer.pack_start(self.error_text, expand=False)
+
+        button_box = gtk.Alignment(xalign=0.5)
+        button_box.show()
+        footer.pack_start(button_box, expand=False)
+        self.cancel_button = gtk.Button(stock=gtk.STOCK_STOP)
+        button_box.add(self.cancel_button)
 
         self.connect('realize', self.__realize_cb)
 
@@ -66,7 +95,9 @@ class LaunchWindow(gtk.Window):
     def __size_changed_cb(self, screen):
         self._update_size()
 
+
 class LaunchBox(hippo.CanvasBox):
+
     def __init__(self, activity_id, icon_path, icon_color):
         gobject.GObject.__init__(self, orientation=hippo.ORIENTATION_VERTICAL)
 
@@ -110,7 +141,9 @@ class LaunchBox(hippo.CanvasBox):
         else:
             self._activity_icon.props.paused = True
 
+
 class _Animation(animator.Animation):
+
     def __init__(self, icon, start_size, end_size):
         animator.Animation.__init__(self, 0.0, 1.0)
 
@@ -122,7 +155,6 @@ class _Animation(animator.Animation):
         d = (self.end_size - self.start_size) * current
         self._icon.props.size = int(self.start_size + d)
 
-_launchers = {}
 
 def setup():
     model = shell.get_model()
@@ -130,31 +162,56 @@ def setup():
     model.connect('launch-failed', __launch_failed_cb)
     model.connect('launch-completed', __launch_completed_cb)
 
-def add_launcher(activity_id, icon_path, icon_color):
 
-    if activity_id in _launchers:
+def add_launcher(activity_id, icon_path, icon_color):
+    model = shell.get_model()
+
+    if model.get_launcher(activity_id) is not None:
         return
 
     launch_window = LaunchWindow(activity_id, icon_path, icon_color)
     launch_window.show()
 
-    _launchers[activity_id] = launch_window
+    model.register_launcher(activity_id, launch_window)
+
 
 def __launch_started_cb(home_model, home_activity):
-    add_launcher(home_activity.get_activity_id(), home_activity.get_icon_path(),
-                 home_activity.get_icon_color())
+    add_launcher(home_activity.get_activity_id(),
+            home_activity.get_icon_path(), home_activity.get_icon_color())
+
 
 def __launch_failed_cb(home_model, home_activity):
+    activity_id = home_activity.get_activity_id()
+    launcher = shell.get_model().get_launcher(activity_id)
+
+    if launcher is None:
+        logging.error('Launcher for %s is missing', activity_id)
+    else:
+        launcher.error_text.props.label = _('<b>%s</b> failed to start.') % \
+                home_activity.get_activity_name()
+        launcher.error_text.show()
+
+        launcher.cancel_button.connect('clicked',
+                __cancel_button_clicked_cb, home_activity)
+        launcher.cancel_button.show()
+
+
+def __cancel_button_clicked_cb(button, home_activity):
     _destroy_launcher(home_activity)
 
+
 def __launch_completed_cb(home_model, home_activity):
     _destroy_launcher(home_activity)
 
+
 def _destroy_launcher(home_activity):
     activity_id = home_activity.get_activity_id()
 
-    if activity_id in _launchers:
-        _launchers[activity_id].destroy()
-        del _launchers[activity_id]
-    else:
-        logging.error('Launcher for %s is missing', activity_id)
+    launcher = shell.get_model().get_launcher(activity_id)
+    if launcher is None:
+        if not home_activity.is_journal():
+            logging.error('Launcher was not registered for %s', activity_id)
+        return
+
+    shell.get_model().unregister_launcher(activity_id)
+    launcher.destroy()
