@@ -15,7 +15,6 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 """D-bus service providing access to the shell's functionality"""
-import os
 
 import dbus
 import gtk
@@ -84,7 +83,7 @@ class UIService(dbus.service.Object):
 
         if activity is not None and activity.get_window() is not None:
             activity.get_window().activate(gtk.get_current_event_time())
-            return not activity.props.launching
+            return self._shell_model.get_launcher(activity_id) is None
 
         return False
 
